﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using Microsoft.Drawing;

namespace StudentLibrary
{
    public static class GraphicsUtil
    {

        public static Color SmoothAndGetAverageColor_Sequential(Bitmap img)
        {
            // Smooth the image first
            Smooth_Sequential(img);

            // Then get the average of the smoothed image
            return GetAverageColor(img);
        }

        public static Color SmoothAndGetAverageColor_Parallel(Bitmap img)
        {
            // TODO: You should implement a parallelized version of this method.
            return SmoothAndGetAverageColor_Sequential(img);
        }

        /// <summary>
        /// Smoothes the image using the neighboring pixels of each pixel.
        /// Border cases are left alone.
        /// </summary>
        /// <param name="img"></param>
        /// <returns></returns>
        public static void Smooth_Sequential(Bitmap img)
        {
            // Create a copy of the original image to read from
            Bitmap origImg = (Bitmap)img.Clone();

            unsafe
            {
                FastBitmap fastImg = null;
                FastBitmap fastOrigImg = null;
                try
                {
                    // Create FastBitmap versions of the base images for faster access
                    fastImg = new FastBitmap(img);
                    fastOrigImg = new FastBitmap(origImg);

                    // Iterate thru each row and column
                    int rowBound = img.Height - 1;
                    int colBound = img.Width - 1;
                    for (int y = 1; y < rowBound; y++)
                    {
                        for (int x = 1; x < colBound; x++)
                        {
                            int r = 0, g = 0, b = 0;
                            PixelData* px;

                            // Add up the pixel values of the neighbors
                            px = fastOrigImg[x - 1, y - 1]; r += px->R; g += px->G; b += px->B;
                            px++; r += px->R; g += px->G; b += px->B;
                            px++; r += px->R; g += px->G; b += px->B;
                            px = fastOrigImg[x - 1, y + 0]; r += px->R; g += px->G; b += px->B;
                            px++; r += px->R; g += px->G; b += px->B;
                            px++; r += px->R; g += px->G; b += px->B;
                            px = fastOrigImg[x - 1, y + 1]; r += px->R; g += px->G; b += px->B;
                            px++; r += px->R; g += px->G; b += px->B;
                            px++; r += px->R; g += px->G; b += px->B;

                            // Set the color to the average
                            *fastImg[x, y] = new PixelData() {
                                R = (byte)Math.Round(r / 9.0f),
                                G = (byte)Math.Round(g / 9.0f),
                                B = (byte)Math.Round(b / 9.0f),
                            };
                        }
                    }
                }
                finally
                {
                    // Be sure to free our resources
                    if (fastImg != null) fastImg.Dispose();
                    if (fastOrigImg != null) fastOrigImg.Dispose(); // Need to dispose this BEFORE disposing of it's image
                    if (origImg != null) origImg.Dispose();
                }
            }
        }

        /// <summary>
        /// Computes the average color of the specified image.
        /// </summary>
        /// <param name="img"></param>
        /// <returns></returns>
        public static Color GetAverageColor(Bitmap img)
        {
            unsafe
            {
                FastBitmap fastImg = null;
                try
                {
                    // Create FastBitmap versions of the base images for faster access
                    fastImg = new FastBitmap(img);

                    // Iterate thru each row and column
                    // For each row, get its average color
                    // then compute the average of row averages.
                    int height = img.Height;
                    int width = img.Width;
                    long rowsR = 0, rowsG = 0, rowsB = 0;
                    for (int y = 0; y < height; y++)
                    {
                        int rowR = 0, rowG = 0, rowB = 0;
                        for (int x = 0; x < width; x++)
                        {
                            PixelData px = *fastImg[x, y];
                            rowR += px.R; rowG += px.G; rowB += px.B;
                        }
                        rowsR += rowR;
                        rowsG += rowG;
                        rowsB += rowB;
                    }

                    long pxCnt = height * width;
                    Color avgColor = Color.FromArgb(
                        (int)Math.Round((double)rowsR / pxCnt),
                        (int)Math.Round((double)rowsG / pxCnt),
                        (int)Math.Round((double)rowsB / pxCnt)
                        );

                    return avgColor;
                }
                finally
                {
                    // Be sure to free our resources
                    if (fastImg != null) fastImg.Dispose();
                }
            }
        }

    }
}
